/**
 * @file
 * Allows a user to save their Instagram user ID.
 */

// Configure JS Lint.
/*jslint browser: true, white: true */
/*global jQuery, Drupal */


// Initializes Drupal.instagramProfile.
Drupal.instagramProfile = Drupal.instagramProfile || {
  behaviors: []
};

// Sets the text label of the configure/authenticate link.
Drupal.instagramProfile.behaviors.authenticate = function ($list, id) {
  "use strict";
  var label = id
    ? Drupal.t('Share a different Instagram user\u2019s feed')
    : Drupal.t('Share your Instagram feed');

  $list.siblings('a.authenticate').text(label);
};

// Appends a button to disable the shared Instagram feed.
Drupal.instagramProfile.behaviors.disable = function ($list, id) {
  "use strict";
  var label, $button;

  // Check there is:
  //   - a shared feed to disable
  //   - not already a disable button
  if (id && $list.parent().find('button.disable').length < 1) {
    label = Drupal.t('Stop showing your Instagram feed');
    $button = jQuery('<button class="disable">' + label + '</button>');

    // Add the button in a wrapper.
    jQuery('<div></div>').append($button).insertBefore($list);

    // When the button is clicked;
    $button.bind('click', function () {
      // Delete the saved user ID in the database.
      Drupal.instagramProfile.userIdApi('DELETE', {}, function () {
        // Reload the page (and reset state) once the user ID has been saved.
        location.reload();
      });
    });
  }
};

// Accesses the server-side Instagram user ID API resource/endpoint.
Drupal.instagramProfile.userIdApi = function (method, data, callback) {
  "use strict";
  // Initialize the data variable.
  data = data || {};
  // Include the CSRF token for all API requests.
  data.csrf_token = Drupal.settings.instagramProfile.csrfToken;

  jQuery.ajax({
    type:    method,
    // Support Drupal installed in a sub-directory.
    url:     Drupal.settings.basePath + 'instagram_profile/user_id',
    data:    data,
    success: callback
  });
};

// Shows throbbers for all lists/feeds in the page.
Drupal.instagramProfile.showThrobbers = function (context) {
  "use strict";
  Drupal.instagramProfile.lists(context).children().show();
};

// Saves user's own Instagram user ID after authenticating on Instagram.com.
// Bind this callback to document-ready instead Drupal.behaviors since it does
// not bind to any DOM and should only run once in the life of a page.
jQuery(function () {
  "use strict";
  var accessToken,
    // Functions.
    main, getAccessToken, saveUserId,
    // The key of the access token in the URL's hash.
    key = 'access_token=';

  // Gets the access token, then saves it to the server/database, if found.
  main = function () {
    accessToken = getAccessToken();
    if (accessToken) {
      // Show the throbbers (which were hid by
      // Drupal.instagramProfile.behaviors.view()), when the user is
      // authenticating.
      Drupal.instagramProfile.showThrobbers(document);

      // Get the current user's Instagram user object.
      Drupal.instagramProfile.get('users/self', saveUserId, { access_token: accessToken });

      // Remove the access token from the URL so that they do not accidentally
      // share it by copying and pasting the URL.
      if (history.replaceState) {
        // If the browser supports History API, update history state to exclude
        // the access token.
        history.replaceState(null, document.title, location.href.replace('#' + key + accessToken, ''));
      }
      else {
        // Reset the hash.  (This leaves the hash character (`#`) in the URL.)
        window.location.hash = '';
      }
    }
  };

  // Looks for and retrieves the access token in the URL's hash.
  getAccessToken = function() {
    var part,
      // Split the hash by the ampersand character.
      parts = window.location.hash.substr(1).split('&');

    // Iterate over each section of the hash.
    while (parts.length) {
      part = parts.pop();
      // If this part starts with the access token's key: "access_token=";
      if (part.indexOf(key) === 0) {
        // Retrieve the value.
        return part.substr(key.length);
      }
    }
  };

  // Saves the Instagram user ID to the database and updates the UI.
  saveUserId = function (response) {
    Drupal.instagramProfile.attachBehaviors(document, parseInt(response.data.id, 10));
    Drupal.instagramProfile.userIdApi('POST', { instagram_user_id: response.data.id });
  };

  // Start execution.
  main();
});
