/**
 * @file
 * Allows a user to save their Instagram user ID.
 */

// Configure JS Lint.
/*jslint white: true */
/*global jQuery, Drupal */


// When Drupal attaches it's behaviors;
Drupal.behaviors.instagramProfile = {
  attach: function (context) {
    "use strict";
    // Attach instagramProfile's behaviors.
    Drupal.instagramProfile.attachBehaviors(context);
  }
};

// Initializes Drupal.instagramProfile.
Drupal.instagramProfile = Drupal.instagramProfile || {
  behaviors: []
};

// Gets data from the Instagram API.
Drupal.instagramProfile.get = function (resource, callback, data) {
  "use strict";

  // Use JSONP, since Instagram has not implemented CORS.
  var url = 'https://api.instagram.com/v1/' + resource + '/?callback=?';

  // Initialize the data variable.
  data = data || {};

  // Specify a client ID if an access token has not been provided.  Most
  // resource endpoints accept just a client ID.  Some, such as `users/self`,
  // require an access token to identify the "current" user ("self").
  if (!data.accessToken) {
    data.client_id = Drupal.settings.instagramProfile.clientId;
  }

  jQuery.getJSON(url, data, callback);
};

// Attaches instagramProfile's behaviors to instagram-profile feeds/lists.
Drupal.instagramProfile.attachBehaviors = function (context, newId) {
  "use strict";
  var behaviors = Drupal.instagramProfile.behaviors;

  // Get any feeds/lists on the page and iterate over them.
  Drupal.instagramProfile.lists(context).each(function () {
    var $list, oldId, i;

    // This feed/list.
    $list = jQuery(this);

    // The current Instagram user ID of the feed.
    oldId = $list.data('instagram-id');

    // If the new ID is different to the old one;
    if (newId !== oldId) {
      if (newId) {
        // Update the list's stored ID for future reference.
        $list.data('instagram-id', newId);
      }

      // Invoke all behaviors with the current ID.
      for (i in behaviors) {
        if (behaviors.hasOwnProperty(i)) {
          behaviors[i]($list, newId || oldId, Drupal.settings.instagramProfile);
        }
      }
    }
  });  
};

// Gets the feeds/lists in a document/element.
Drupal.instagramProfile.lists = function (context) {
  "use strict";
  return jQuery(context).find('ul.instagram-profile-feed');
};

// Loads an Instagram feed to view it's media items.
Drupal.instagramProfile.behaviors.view = function ($list, id) {
  "use strict";
  if (!id) {
    // Hide the throbber if this list has no ID nor is there an access token.
    $list.children().hide();
  }
  else {
    // Retrieve the feed.
    Drupal.instagramProfile.get('users/' + id + '/media/recent', function (response) {
      // Once retrieved;
      // Hide the throbber.
      $list.children().hide();

      // Render and append the Instagram media items, most recent first.
      while (response.data.length) {
        // Most recent items are at the end of the feed data.
        $list.append('<li>' + Drupal.theme('instagramItem', response.data.shift()) + '</li>');
      }
    });
  }
};

//
// Default theme implementation for an instagram media item.
//
// Override this in custom javascript with:
// Drupal.theme.instagramItem = function(item, size) {
//   // ...
// };
//
Drupal.theme.prototype.instagramItem = function (item, size) {
  "use strict";
  var image, alt,
      markup = '',
      user = item.user,
      // Not all media items have a caption property.
      caption = item.caption ? item.caption.text : '';

  // Default to low resolution images.
  size = size || 'low';
  size += '_resolution';
  image = item.images[size];

  // The image's alt="" attribute.
  alt = Drupal.t('@name&rsquo;s photo', { '@name': user.full_name });

  // The markup.  A real templating engine would make this much cleaner.
  markup += '<div class="profile">';
  markup += '  <img src="' + user.profile_picture + '" alt="' + alt + '" title="' + user.username + '" />';
  markup += '  <span class="name" title="' + user.username + '">' + user.full_name + '</span>';
  markup += '</div>';

  markup += '<img class="thumbnail" src="' + image.url + '" alt="' + caption + '" height="' + image.height + '" width="' + image.width + '" />';

  markup += '<p class="likes">' + item.likes.count + ' Likes</p>';
  markup += '<p class="caption">' + caption + '</p>';

  return markup;
};
